#include "thumbnailmanager.h"

#include <QX11Info>
#include <X11/Xutil.h>
#include <QStandardPaths>
#include <QFile>
#include <QBitmap>

QImage qimageFromXImage(XImage* ximage)
{
    QImage::Format format = QImage::Format_ARGB32_Premultiplied;
    if (ximage->depth == 24)
        format = QImage::Format_RGB32;
    else if (ximage->depth == 16)
        format = QImage::Format_RGB16;

    QImage image = QImage(reinterpret_cast<uchar*>(ximage->data),
                          ximage->width, ximage->height,
                          ximage->bytes_per_line, format).copy();

    // 大端还是小端?
    if ((QSysInfo::ByteOrder == QSysInfo::LittleEndian && ximage->byte_order == MSBFirst)
            || (QSysInfo::ByteOrder == QSysInfo::BigEndian && ximage->byte_order == LSBFirst)) {

        for (int i = 0; i < image.height(); i++) {
            if (ximage->depth == 16) {
                ushort* p = reinterpret_cast<ushort*>(image.scanLine(i));
                ushort* end = p + image.width();
                while (p < end) {
                    *p = ((*p << 8) & 0xff00) | ((*p >> 8) & 0x00ff);
                    p++;
                }
            } else {
                uint* p = reinterpret_cast<uint*>(image.scanLine(i));
                uint* end = p + image.width();
                while (p < end) {
                    *p = ((*p << 24) & 0xff000000) | ((*p << 8) & 0x00ff0000)
                         | ((*p >> 8) & 0x0000ff00) | ((*p >> 24) & 0x000000ff);
                    p++;
                }
            }
        }
    }

    // 修复alpha通道
    if (format == QImage::Format_RGB32) {
        QRgb* p = reinterpret_cast<QRgb*>(image.bits());
        for (int y = 0; y < ximage->height; ++y) {
            for (int x = 0; x < ximage->width; ++x)
                p[x] |= 0xff000000;
            p += ximage->bytes_per_line / 4;
        }
    }

    return image;
}

bool isBlackBitmap(const QImage &image)
{
    if(image.isNull()) return true;

    int num = 0;
    for(int i = 0; i < image.width(); i++)
    {
        for(int j = 0; j < image.height();j++)
        {
            QRgb rgb = image.pixel(i,j);
            if(qRed(rgb) <= 120 && qGreen(rgb) <= 120 && qBlue(rgb) <= 120)
                num++;
        }
    }
    float rate = (float)num / (float)(image.width()*image.height());
    //阈值超过0.99777表示全黑
    if(rate >= 0.99777)
        return true;

    return  false;
}

ThumbnailManager::ThumbnailManager(QObject *parent) : QObject(parent)
{
    //
    connect(KWindowSystem::self(), &KWindowSystem::windowAdded, this, &ThumbnailManager::onKwinWindowAdd);
    connect(KWindowSystem::self(), &KWindowSystem::windowRemoved, this, &ThumbnailManager::onKwinWindowRemove);
    connect(KWindowSystem::self(),
            static_cast<void (KWindowSystem::*)(WId, NET::Properties, NET::Properties2)>(&KWindowSystem::windowChanged)
            , this, &ThumbnailManager::onKwinWindowChanged);

    onBackgroundChanged();
}

ThumbnailManager *ThumbnailManager::getIns()
{
    static ThumbnailManager tm;
    return &tm;
}

QPixmap ThumbnailManager::getThumbnail(WId id)
{
    QPixmap pixmap;
    if(m_thumbnails.contains(id))
    {
        pixmap = m_thumbnails.value(id);
    }else {
        //这里如果获取不到，就用背景替代
        pixmap = getWallPaper();
    }
    return pixmap;
}

QPixmap ThumbnailManager::getWallPaper()
{
    onBackgroundChanged();
    return m_wallPaper;
}

void ThumbnailManager::onKwinWindowAdd(WId id)
{
    updateThumbnail(id);
    emit windowAdded(id);
}

void ThumbnailManager::onKwinWindowRemove(WId id)
{
    removeThumbnail(id);
    emit windowRemoved(id);
}

void ThumbnailManager::onKwinWindowChanged(WId id, NET::Properties p1, NET::Properties2 p2)
{
    updateThumbnail(id);
    emit windowChanged(id,p1,p2);
}

void ThumbnailManager::onBackgroundChanged()
{
    QString path = getBackgroundSettingPath();
    QSettings m_wallPaperSetting(path, QSettings::IniFormat, this);
    m_wallPaperSetting.beginGroup("Desktop");
    QString defaultwallpaper = "/usr/share/backgrounds/default.png";
    QString wallpaper = m_wallPaperSetting.value("Wallpaper", defaultwallpaper).toString();
    m_wallPaperSetting.endGroup();

    //不存在壁纸文件就使用 壁纸cache文件
    if(!QFile::exists(wallpaper)) {
        QString cacheFileName = QString::fromLocal8Bit(qgetenv("XDG_CACHE_HOME"));
        if(cacheFileName.isEmpty()) {
            cacheFileName = QStandardPaths::writableLocation(QStandardPaths::HomeLocation) % QLatin1String("/.cache");
        }
        cacheFileName += QLatin1String("/nde-fileman/nde/wallpaper.cache");
        wallpaper = cacheFileName;
        //cache 文件也不存在，就使用默认壁纸
        if(!QFile::exists(wallpaper)) wallpaper = defaultwallpaper;
    }

    if(wallpaper.compare(m_wallPaperPath) == 0) return;

    m_wallPaperPath = wallpaper;
    m_wallPaper.load(wallpaper);
}

void ThumbnailManager::removeThumbnail(WId id)
{
    m_thumbnailsLock.lock();
    if(m_thumbnails.contains(id))
    {
        m_thumbnails.remove(id);
    }

    m_thumbnailsLock.unlock();
}

void ThumbnailManager::updateThumbnail(WId id)
{
    const auto display = XOpenDisplay(getenv("DISPLAY"));

    XWindowAttributes attr;
    XGetWindowAttributes(display, id, &attr);

    XImage *ximage = XGetImage(display, id, 0, 0, attr.width, attr.height, ~0, ZPixmap);

    if(display)
    {
        XCloseDisplay(display);
    }

    if(ximage)
    {
//        QImage qImage = QImage((const uchar*)(ximage->data), ximage->width, ximage->height, ximage->bytes_per_line, QImage::Format_RGB32);
        QImage thumbnail = qimageFromXImage(ximage);

        if(!isBlackBitmap(thumbnail))
        {
            m_thumbnailsLock.lock();
            m_thumbnails.insert(id, QPixmap::fromImage(thumbnail));
            m_thumbnailsLock.unlock();
        }
        XDestroyImage(ximage);
    }
}

QString ThumbnailManager::getBackgroundSettingPath()
{
    QString dirName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
    dirName += "/nde-fileman/nde/settings.conf";
    return dirName;
}
