#include "displaywidget.h"
#include <KWindowEffects>
#include <QBitmap>
#include <QPainter>
#include <QEvent>
#include <QKeyEvent>
#include <QApplication>
#include <QScreen>
#include <QDesktopServices>
#include "appmanager.h"
#include <QDebug>
#include <QPainterPath>

DisplayWidget::DisplayWidget(INdePanelPlugin *plugin, QWidget *parent)
    : QWidget(parent),
      mPlugin(plugin),
      mOpacity(-1),
      mMenuDisplayMode(DisplayNormal)
{
    this->setObjectName("DisplayWidget");
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);
    KWindowEffects::enableBlurBehind(this->winId(), true);

    mDisplayMainMenu = new DisplayMainMenu(this);
    connect(mDisplayMainMenu, &DisplayMainMenu::signalAddToDesktop,this,&DisplayWidget::slotAddToDesktop);
    connect(mDisplayMainMenu, &DisplayMainMenu::signalAddToQuickLaunch,this,&DisplayWidget::slotAddToQuickLaunch);
    mDisplayMainMenuFull = new DisplayMainMenuFull(this);
    connect(mDisplayMainMenuFull, &DisplayMainMenuFull::signalAddToDesktop,this,&DisplayWidget::slotAddToDesktop);
    connect(mDisplayMainMenuFull, &DisplayMainMenuFull::signalAddToQuickLaunch,this,&DisplayWidget::slotAddToQuickLaunch);
    connect(AppManager::getIns(), &AppManager::menuDisplayModeChanged,this,&DisplayWidget::slotMainMenuModeSwitch);

    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setSpacing(0);
    layout->setContentsMargins(0,0,0,0);
    layout->addWidget(mDisplayMainMenu);
    layout->addWidget(mDisplayMainMenuFull);
    this->setLayout(layout);

    mAnimation = new QPropertyAnimation(this, "geometry");
    mAnimation->setEasingCurve(QEasingCurve::Linear);
    mAnimation->setDuration(100);
    connect(mAnimation, &QPropertyAnimation::finished, this, &DisplayWidget::slotModeSwitchAnimFinished);
}

void DisplayWidget::showMainMenu()
{
    AppManager::getIns()->restoreMenuDisplayMode();
    mMenuDisplayMode = AppManager::getIns()->menuDisplayMode();

    showMainMenuByMode();
}

void DisplayWidget::realign()
{
    int opacity = mPlugin->panel()->opacity();
    QColor bColor = mPlugin->panel()->backgroundColor();
    if(mOpacity == opacity && mBackgroundColor == bColor )
        return;

    mOpacity = opacity;
    mBackgroundColor = bColor;

    QString color = QString("%1, %2, %3, %4")
        .arg(mBackgroundColor.red())
        .arg(mBackgroundColor.green())
        .arg(mBackgroundColor.blue())
        .arg(mOpacity / 100.0);
    this->setStyleSheet(QString("#DisplayWidget { "
                                "background-color: rgba(" + color + "); }"));

//    this->hide();

//    mDisplayMainMenu->setGradientColor(QColor(mBackgroundColor.red(),mBackgroundColor.green(),mBackgroundColor.blue(), mOpacity));
    //    mDisplayMainMenuFull->setGradientColor(QColor(mBackgroundColor.red(),mBackgroundColor.green(),mBackgroundColor.blue(), mOpacity));
}

void DisplayWidget::slotMainMenuModeSwitch(MenuDisplayMode mode)
{
    if(mMenuDisplayMode == mode) return;

    if(mAnimation->state() == QPropertyAnimation::Running)
    {
        mAnimation->pause();
    }
    QRect oldRect = calGeometry();
    mMenuDisplayMode = mode;
    QRect newRect = calGeometry();
    mAnimation->setStartValue(oldRect);
    mAnimation->setEndValue(newRect);

    //animation start
    mDisplayMainMenu->hide();
    mDisplayMainMenuFull->hide();
    mAnimation->start();
}

void DisplayWidget::slotModeSwitchAnimFinished()
{
    showMainMenuByMode();
}

void DisplayWidget::slotAddToDesktop(XdgDesktopFile desktopFile)
{
    QString desktopPath = QStandardPaths::writableLocation(QStandardPaths::DesktopLocation);
    //直接保存destop文件到桌面可能会遇到无法打开的问题。
    //desktopFile.save(desktopPath +"/"+ XdgDesktopFile::id( desktopFile.fileName(),false));
    QString tmp = QString("cp %1 %2").arg(desktopFile.fileName()).arg(desktopPath +"/"+ XdgDesktopFile::id( desktopFile.fileName()));
    QProcess::startDetached(tmp);
}

void DisplayWidget::slotAddToQuickLaunch(XdgDesktopFile desktopFile)
{
    NdePanel *panel = static_cast<NdePanel *>( mPlugin->panel() );
    if(panel)
    {
        const INdePanelPlugin * c_i_plugin =  panel->getPlugin("QuickLaunch");
        INdePanelPlugin * i_plugin = const_cast<INdePanelPlugin *>(c_i_plugin);
        NdeQuickLaunchPlugin * plugin = dynamic_cast<NdeQuickLaunchPlugin *>(i_plugin);
        if(plugin)
        {
            NdeQuickLaunch * widget = qobject_cast<NdeQuickLaunch*>( plugin->widget());
            if(widget)
            {
                widget->addButtonQuick(desktopFile);
            }
        }
    }
}

bool DisplayWidget::event(QEvent *event)
{
    if(event->type() == QEvent::KeyPress ||
            event->type() == QEvent::KeyRelease)
    {
        QKeyEvent* ke = static_cast<QKeyEvent*>(event);
        if(ke && ke->key() == Qt::Key_Escape)
        {
            this->hide();
        }
    }
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            //这里需要延时隐藏界面，由于快捷键一按下就会触发此事件，界面就会出现闪烁。
            QTimer::singleShot(200, this, [this](){this->hide();});
        }
    }
    return QWidget::event(event);
}

void DisplayWidget::paintEvent(QPaintEvent *event)
{
    if(mMenuDisplayMode == DisplayNormal)
    {
        QBitmap bmp(this->size());
        bmp.fill();
        QPainter p(&bmp);
        p.setRenderHint(QPainter::Antialiasing);
        p.setPen(Qt::transparent);
        p.setBrush(mBackgroundColor);//此行代码必须添加，不然无法达到正常的显示
        //p.drawRoundedRect(bmp.rect(), 20, 20); //四个角都是圆弧
        int arcR = 10; //圆弧大小
        QRect rect = this->rect().adjusted(0,0,0,0);
        QPainterPath path;
        switch(mPlugin->panel()->position())
        {
        case NdePanel::PositionBottom:
            path.moveTo(0, 0);
            path.lineTo(0, rect.height());
            path.lineTo(rect.width(), rect.height());
            path.lineTo(rect.width(), arcR);
            path.arcTo(rect.width() - arcR*2, 0, arcR * 2, arcR * 2, 0, 90);
            path.lineTo(0,0);
            break;
        case NdePanel::PositionTop:
        case NdePanel::PositionLeft:
            path.moveTo(0, 0);
            path.lineTo(0, rect.height());
            path.lineTo(rect.width() - arcR, rect.height());
            path.arcTo(rect.width() - arcR*2, rect.height() - arcR*2, arcR * 2, arcR * 2, 270, 90);
            path.lineTo(rect.width(), 0);
            path.lineTo(0,0);
            break;
        case NdePanel::PositionRight:
            path.moveTo(0, 0);
            path.lineTo(0, rect.height() - arcR);
            path.arcTo(0, rect.height() - arcR*2, arcR * 2, arcR * 2, 180, 90);
            path.lineTo(rect.width(), rect.height());
            path.lineTo(rect.width(), 0);
            path.lineTo(0,0);
            break;
        }

        p.drawPath(path);

        this->setMask(bmp);
    }
    else
    {
        this->clearMask();
    }
}

void DisplayWidget::showMainMenuByMode()
{
    if(mMenuDisplayMode == DisplayNormal)
    {
        mDisplayMainMenuFull->hide();
        mDisplayMainMenu->show();
        mDisplayMainMenu->restoreUI();
    }
    else if(mMenuDisplayMode == DisplayFull)
    {
        mDisplayMainMenu->hide();
        mDisplayMainMenuFull->show();
        mDisplayMainMenuFull->restoreUI();
    }
    this->setGeometry(calGeometry());
    this->show();
}

QRect DisplayWidget::calGeometry()
{
    QRect rect;
    QRect availableRect = qApp->primaryScreen()->availableGeometry();

    if(mMenuDisplayMode == DisplayFull)
    {
        rect = availableRect;
    }
    else
    {
        QSize fixedSize = QSize(qMin(availableRect.width(), 374)  ,qMin(availableRect.height(), 458));
        rect = mPlugin->calculatePopupWindowPos(fixedSize);
    }
    return rect;
}
