#include "notificationbar.h"
#include <QMouseEvent>
#include <QHBoxLayout>
#include <QScreen>
#include "notifyd.h"
#include <QtDBus/QDBusConnection>
#include "notificationsadaptor.h"
#include <QDebug>

NotificationBarWidget::NotificationBarWidget(INdePanelPlugin *plugin, QWidget *parent) :
    QFrame(parent),
    mPlugin(plugin)
{
    mDisplayWidth = NOTIFICATIONBAR_WIDTH;
    mIsDisplayShow = false;

    QHBoxLayout *layout = new QHBoxLayout(this);
    layout->setContentsMargins (0, 0, 0, 0);

    //btn
    mBtn = new QToolButton(this);
    mBtn->setAutoRaise(true);
    mBtn->setIcon(QIcon(":/res/notificationbar.png"));
    mBtn->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    connect(mBtn, &QToolButton::clicked, this, &NotificationBarWidget::onBtnClicked);

    //default
    QRect rc = qApp->primaryScreen()->availableGeometry();
    mDisplay = new NotifyDisplayWidget(plugin);
    mDisplay->setGeometry(rc.adjusted(rc.width(),0,mDisplayWidth,0));
    mDisplay->hide();

    connect(mDisplay, &NotifyDisplayWidget::lostFocus,
            this, &NotificationBarWidget::onDisplayLostFocus);
    connect(mDisplay, &NotifyDisplayWidget::signalNotifiesCountChanged,
            this, &NotificationBarWidget::onNotifiesCountChanged);

    //animation
    mAnimation = new QPropertyAnimation(mDisplay, "geometry", mDisplay);
    mAnimation->setDuration(200);
    mAnimation->setEasingCurve(QEasingCurve::Linear);
    mAnimation->setStartValue(rc.adjusted(rc.width(),0,mDisplayWidth,0));
    mAnimation->setEndValue(rc.adjusted(rc.width()-mDisplayWidth,0,0,0));
    connect(mAnimation, &QPropertyAnimation::finished, this, &NotificationBarWidget::onAnimationFinished);

    layout->addWidget(mBtn);
    setLayout(layout);

    //add dbus service
    Notifyd* daemon = new Notifyd(mDisplay, this);
    new NotificationsAdaptor(daemon);

    QDBusConnection connection = QDBusConnection::sessionBus();
    if (!connection.registerService(QSL("org.freedesktop.Notifications")))
        qDebug() << "registerService failed: another service with 'org.freedesktop.Notifications' runs already";
    if (!connection.registerObject(QSL("/org/freedesktop/Notifications"), daemon))
        qDebug() << "registerObject failed: another object with '/org/freedesktop/Notifications' runs already";

    //test
//    for(int i = 0; i < 2; i++)
//    {
//        daemon->Notify(QString("test") + QString::number(i), i+1,
//                       "audio-volume-high",
//                       "this is a test",
//                       "test test test test test test test test test "
//                       "test test test test test test test test test "
//                       "test test test",
//                       QStringList() << "aaa1" << "bbb1" << "aaa2" << "bbb2" << "aaa3" << "bbb3" << "aaa4" << "bbb4",
//                       QVariantMap(), 0);
//    }
}

NotificationBarWidget::~NotificationBarWidget()
{
    QDBusConnection::sessionBus().unregisterService("org.freedesktop.Notifications");

    mAnimation->deleteLater();
    mDisplay->deleteLater();
    mBtn->deleteLater();
}

void NotificationBarWidget::realign()
{
    mDisplay->realign();

    NdePanel *panel = static_cast<NdePanel *>( mPlugin->panel() );
    if(panel)
    {
        INdePanel::Position pos = panel->position();
        QRect rc = qApp->primaryScreen()->geometry();
        //位置变化或者屏幕尺寸变化隐藏该控件
        if(pos != mPanelPos || mScreenRect != rc)
        {
            mPanelPos = pos;
            mScreenRect = rc;

            switch(pos)
            {
            case INdePanel::PositionTop:
                rc.adjust(0,panel->panelSize(),0,0);
                break;
            case INdePanel::PositionBottom:
                rc.adjust(0,0,0,-panel->panelSize());
                break;
            case INdePanel::PositionLeft:
                rc.adjust(0,0,0,0);
                break;
            case INdePanel::PositionRight:
                rc.adjust(0,0,-panel->panelSize(),0);
                break;
            }

            mAnimation->setStartValue(rc.adjusted(rc.width(),0,mDisplayWidth,0));
            mAnimation->setEndValue(rc.adjusted(rc.width()-mDisplayWidth,0,0,0));

            mIsDisplayShow = false;
            mDisplay->hide();
        }
    }
}

void NotificationBarWidget::onDisplayLostFocus()
{
    if(mIsDisplayShow)
    {
        mIsDisplayShow = !mIsDisplayShow;
        if(mAnimation->state() == QPropertyAnimation::Running)
            mAnimation->pause();
        QPropertyAnimation::Direction d = mIsDisplayShow ?
                    QPropertyAnimation::Forward:QPropertyAnimation::Backward;

        mAnimation->setDirection(d);
        mAnimation->start();
    }
}

void NotificationBarWidget::onNotifiesCountChanged(int count)
{
    if(count > 0)
    {
        mBtn->setIcon(QIcon(":/res/notificationbar-notified.png"));
    }
    else
    {
        mBtn->setIcon(QIcon(":/res/notificationbar.png"));
    }
}

void NotificationBarWidget::onBtnClicked()
{
    mIsDisplayShow = !mIsDisplayShow;

    if(mAnimation->state() == QPropertyAnimation::Running)
        mAnimation->pause();
    QPropertyAnimation::Direction d = mIsDisplayShow ?
                QPropertyAnimation::Forward:QPropertyAnimation::Backward;

    mAnimation->setDirection(d);

    //
    if(mIsDisplayShow)
    {
        mPlugin->panel()->willShowWindow(mDisplay);
        mDisplay->show();
        mDisplay->activateWindow();
    }

    mAnimation->start();
}

void NotificationBarWidget::onAnimationFinished()
{
    mDisplay->setVisible(mIsDisplayShow);
}

//-------------------------------

NotificationBar::NotificationBar(
        const INdePanelPluginStartupInfo &startupInfo)
    : QObject(),
      INdePanelPlugin(startupInfo)
{
    mWidget = new NotificationBarWidget(this);
    realign();
}

NotificationBar::~NotificationBar()
{

}

void NotificationBar::realign()
{
    mWidget->realign();
}
