#include "notifydisplaywidget.h"
#include "ui_notifydisplaywidget.h"
#include <QDebug>
#include <KWindowEffects>
#include <QTimer>
#include <QProcess>
#include <QDBusConnection>
#include <QDBusInterface>

NotifyDisplayWidget::NotifyDisplayWidget(INdePanelPlugin *plugin, QWidget *parent) :
    QWidget(parent),
    m_ui(new Ui::NotifyDisplayWidget),
    mPlugin(plugin),
    mOpacity(-1)
{
    m_ui->setupUi(this);
    setObjectName("NotifyDisplayWidget");
    this->setFocusPolicy(Qt::ClickFocus);
    this->setWindowFlags(Qt::FramelessWindowHint |
                         Qt::WindowStaysOnTopHint |
                         Qt::Tool |
                         Qt::X11BypassWindowManagerHint);

    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

    KWindowEffects::enableBlurBehind(this->winId(), true);

    connect(m_ui->btn_clear, &QToolButton::clicked,this,&NotifyDisplayWidget::onClear);
    connect(m_ui->btn_remote, &QToolButton::clicked,this,&NotifyDisplayWidget::onRemote);
    connect(m_ui->btn_setting, &QToolButton::clicked,this,&NotifyDisplayWidget::onSetting);
    connect(m_ui->btn_screen_shot, &QToolButton::clicked,this,&NotifyDisplayWidget::onScreenShot);
    connect(m_ui->btn_multi_screen, &QToolButton::clicked,this,&NotifyDisplayWidget::onMultiScreen);

    mTimer = new QTimer(this);
    mTimer->setInterval(100);
    mTimer->setSingleShot(false);
    connect(mTimer, &QTimer::timeout, this, &NotifyDisplayWidget::onTimeout);

    m_notifyLayout = new QVBoxLayout();
    m_notifyLayout->setMargin(0);
    m_notifyLayout->setContentsMargins(0,0,0,0);
    m_notifyLayout->setSpacing(20);
    m_notifyLayout->addStretch();
    m_ui->scrollAreaContents_notify->setLayout(m_notifyLayout);

    //由于该功能未实现，因此先隐藏
    m_ui->btn_remote->hide();
    m_ui->btn_screen_shot->hide();
    m_ui->btn_multi_screen->hide();
    m_ui->btn_setting->hide();
}

NotifyDisplayWidget::~NotifyDisplayWidget()
{
    delete m_ui;
}

void NotifyDisplayWidget::realign()
{
    int opacity = mPlugin->panel()->opacity();
    QColor bColor = mPlugin->panel()->backgroundColor();
    if(mOpacity == opacity && mBackgroundColor == bColor )
        return;

    mOpacity = opacity;
    mBackgroundColor = bColor;

    QString color = QString("%1, %2, %3, %4")
            .arg(mBackgroundColor.red())
            .arg(mBackgroundColor.green())
            .arg(mBackgroundColor.blue())
            .arg(mOpacity / 100.0);
    this->setStyleSheet(QString("#NotifyDisplayWidget {"
                                "background-color: rgba(" + color + "); }"));
}

void NotifyDisplayWidget::onAddNotify(
        uint id,
        const QString &application,
        const QString &summary,
        const QString &body,
        const QString &icon,
        int timeout,
        const QStringList &actions,
        const QVariantMap &hints,
        bool noSave)
{
    //
    int groupWidth = NOTIFICATIONBAR_WIDTH-10-this->layout()->contentsMargins().left() -this->layout()->contentsMargins().right();

    if(m_groups.contains(id))
    {
        m_groups.value(id)->addNotifyItem(application, summary, body, icon, actions, hints);
    }
    else
    {
        NotifyGroup * group = new NotifyGroup(groupWidth, id, application,summary, body, icon, actions, hints, m_ui->scrollAreaContents_notify);
        connect(group, &NotifyGroup::signalGroupClose, this, &NotifyDisplayWidget::onGroupClose);
        connect(group, &NotifyGroup::actionTriggered, this, &NotifyDisplayWidget::actionTriggered);

        mBuffMutex.lock();
        m_buffGroups.append(group);
        mBuffMutex.unlock();

        //10ms后执行
        mTimer->start(10);
    }

    emit signalNotifiesCountChanged(m_groups.count());
}

void NotifyDisplayWidget::onRmoveNotify(uint id, uint reason)
{
    //
    for(int i = 0; i < m_groups.keys().count(); i++)
    {
        uint key = m_groups.keys().at(i);
        NotifyGroup * group = m_groups.value(key);
        if(group && group->applicationID() == id)
        {
            group->onGroupClose();
        }
    }
    emit signalNotifiesCountChanged(m_groups.count());
}

bool NotifyDisplayWidget::event(QEvent *event)
{
    if(event->type() == QEvent::ActivationChange)
    {
        if(!this->isActiveWindow())
        {
            emit lostFocus();
        }
    }
    return QWidget::event(event);
}

void NotifyDisplayWidget::resizeEvent(QResizeEvent *event)
{
    for(int i = 0; i < m_groups.keys().count(); i++)
    {
        uint key = m_groups.keys().at(i);
        NotifyGroup * group = m_groups.value(key);
        if(group)
        {
            int itemWidth = m_ui->scrollArea_notify->maximumViewportSize().width() -10;
            group->setFixedWidth(itemWidth);
        }
    }
    QWidget::resizeEvent(event);
}

void NotifyDisplayWidget::onGroupClose()
{
    NotifyGroup * group = qobject_cast<NotifyGroup*>(sender());
    if(group)
    {
        notifyClosed(group->applicationID(), 2);
        m_groups.remove(group->applicationID());
    }
    emit signalNotifiesCountChanged(m_groups.count());
}

void NotifyDisplayWidget::onClear()
{
    while(m_groups.count())
    {
        NotifyGroup * group = m_groups.begin().value();
        m_groups.erase(m_groups.begin());

        notifyClosed(group->applicationID(), 2);
        if(group->animationFinished()) group->onGroupClose();
    }
    emit signalNotifiesCountChanged(m_groups.count());
}

void NotifyDisplayWidget::onTimeout()
{
    if(m_buffGroups.isEmpty())
    {
        mTimer->stop();
        return;
    }

    for(int i = 0; i < m_groups.keys().count(); i++)
    {
        //当前列表中有添加动画没有执行完的，下一次在添加到layout中
        if(!m_groups.value(m_groups.keys().at(i))->animationFinished())
        {
            return;
        }
    }

    mBuffMutex.lock();
    NotifyGroup * group = m_buffGroups.takeFirst();
    mBuffMutex.unlock();

    m_groups.insert(group->applicationID(), group);

    m_notifyLayout->insertWidget(0,group);
    if(this->isVisible()) group->onGroupShow();

    emit signalNotifiesCountChanged(m_groups.count());
}

void NotifyDisplayWidget::onSetting()
{
    QProcess::startDetached("nde-systemsettings");
    this->hide();
}

void NotifyDisplayWidget::onRemote()
{
    //
}

void NotifyDisplayWidget::onScreenShot()
{
    //
}

void NotifyDisplayWidget::onMultiScreen()
{
    //
}
