#include "notifygroup.h"
#include <QDebug>
#include <KWindowEffects>

#define MASK_HEIGHT 5

NotifyGroup::NotifyGroup(int width, uint id, QString name,
                         const QString &summary,
                         const QString &body,
                         const QString &icon,
                         const QStringList actions,
                         const QVariantMap &hints, QWidget *parent)
    : QWidget(parent),
      m_extend(false),
      m_appID(id)
{
    this->setObjectName("NotifyGroup");
    this->setFocusPolicy(Qt::NoFocus);
    this->setAttribute(Qt::WA_TranslucentBackground);
    this->setAttribute(Qt::WA_NoSystemBackground, false);

//    KWindowEffects::enableBlurBehind(this->winId(), true);

    m_title = new NotifyGroupTitle(width, id, name, summary, body, icon, actions, hints, this);
    connect(m_title, &NotifyGroupTitle::signalGroupClose, this, &NotifyGroup::onGroupClose);
    connect(m_title, &NotifyGroupTitle::signalGroupExtend, this, &NotifyGroup::onGroupExtend);

    m_maskWgt = new QWidget(this);
    m_maskWgt->setObjectName("MaskWgt");
    m_maskWgt->setAttribute(Qt::WA_TranslucentBackground);
    m_maskWgt->setAttribute(Qt::WA_NoSystemBackground, false);
    m_maskWgt->setGeometry(m_title->geometry().adjusted(5,m_title->geometry().height(),-5,MASK_HEIGHT));
    m_maskWgt->hide();

    //提升一下m_title，不然会被m_maskWgt挡住
    m_title->raise();

    //这里应该先设置尺寸，再添加layout，否则会导致item的间距与设置的不一致
    this->setFixedSize(width, calculateHeight(m_extend));

    m_layout = new QVBoxLayout;
    m_layout->setMargin(0);
    m_layout->setContentsMargins(0,0,0,0);
    m_layout->setSpacing(1);
    m_layout->addWidget(m_title);
    m_layout->addStretch();
    this->setLayout(m_layout);

    mTimer = new QTimer(this);
    mTimer->setInterval(100);
    mTimer->setSingleShot(false);
    connect(mTimer, &QTimer::timeout, this, &NotifyGroup::slotTimeout);

    m_animationClose = new QPropertyAnimation(this, "geometry", this);
    m_animationClose->setDuration(200);
    m_animationClose->setEasingCurve(QEasingCurve::InOutCubic);
    connect(m_animationClose, &QPropertyAnimation::finished,
            this, &NotifyGroup::slotAnimationCloseFinished);

    m_animationShow = new QPropertyAnimation(this, "geometry", this);
    m_animationShow->setDuration(200);
    m_animationShow->setEasingCurve(QEasingCurve::InOutCubic);
    connect(m_animationShow, &QPropertyAnimation::finished,
            this, &NotifyGroup::slotAnimationShowFinished);

    m_animationExtend = new QPropertyAnimation(this, "fixedHeight", this);
    m_animationExtend->setDuration(200);
    connect(m_animationExtend, &QPropertyAnimation::finished,
            this, &NotifyGroup::slotAnimationExtendFinished);

    addNotifyItem(name, summary, body, icon, actions, hints);
}

void NotifyGroup::addNotifyItem(const QString name, const QString &summary, const QString &body, const QString &icon, const QStringList actions, const QVariantMap &hints)
{
    //解析actions中的动作(actions<<acion<<desc，删除动作的描述
    QStringList newActions;
    for (int i = 0; i < actions.count(); i += 2)
    {
        QString key = actions[i];
        newActions.append(key);
    }
    if(newActions.isEmpty()) newActions.append(QString());
    for(int i = 0; i < newActions.count(); i++)
    {
        NotifyItem * item = new NotifyItem(this->width(),m_appID,name,summary,body,icon,newActions.at(i),hints,this);
        connect(item, &NotifyItem::signalClose, this, &NotifyGroup::onItemClose);
        connect(item, &NotifyItem::actionTriggered, this, &NotifyGroup::actionTriggered);
        connect(item, &NotifyItem::signalExpend, this, &NotifyGroup::onItemExpend);

        m_buffItems.append(item);
    }
    if(!m_buffItems.isEmpty()) mTimer->start(10);
}

bool NotifyGroup::animationFinished() const
{
    return (m_animationClose->state() != QPropertyAnimation::Running &&
            m_animationShow->state() != QPropertyAnimation::Running &&
            m_animationExtend->state() != QPropertyAnimation::Running );
}

void NotifyGroup::onItemExpend(bool extend)
{
    //extend是item展开的状态值

    this->setFixedHeight(calculateHeight(m_extend));
}

void NotifyGroup::onItemClose()
{
    NotifyItem * item = qobject_cast<NotifyItem*>(sender());
    if(item)
    {
        m_items.removeOne(item);
    }

    this->setFixedHeight(calculateHeight(m_extend));
    m_title->setNotifyCount(m_items.count());

    if(m_items.count() == 1)
    {
        m_title->hide();
        m_maskWgt->hide();
        m_items.at(0)->show();
    }
    else if(m_items.count() == 0)
    {
        this->onGroupClose();
    }
}

void NotifyGroup::onGroupShow()
{
    if(m_animationShow->state() == QPropertyAnimation::Running)
    {
        m_animationShow->pause();
    }
    QRect r = this->geometry();
    m_animationShow->setStartValue(r.adjusted(-r.width(), 0,-r.width(),0));
    m_animationShow->setEndValue(r);
    m_animationShow->start();
}

void NotifyGroup::onGroupClose()
{
    if(m_animationClose->state() == QPropertyAnimation::Running)
    {
        m_animationClose->pause();
    }
    QRect r = this->geometry();
    m_animationClose->setStartValue(r);
    m_animationClose->setEndValue(r.adjusted(-r.width(), 0,-r.width(),0));
    m_animationClose->start();
}

void NotifyGroup::onGroupExtend(bool extend)
{
    m_extend = extend;

    int heightExpend =  calculateHeight(true);
    int height = calculateHeight(false);

    if(m_animationExtend->state() == QPropertyAnimation::Running)
    {
        m_animationExtend->pause();
//        m_animationExtend->setDirection(m_extend ? QPropertyAnimation::Forward:QPropertyAnimation::Backward);
//        m_animationExtend->resume();
    }
    m_animationExtend->setStartValue(height);
    m_animationExtend->setEndValue(heightExpend);
    m_animationExtend->setDirection(m_extend ? QPropertyAnimation::Forward:QPropertyAnimation::Backward);
    m_animationExtend->start();

//    //从上到下便利子项目

    for(int i= m_items.count()-1; i >= 0; i--)
    {
        NotifyItem * item = m_items.at(i);
        height += m_layout->spacing();
        item->slotMove(m_extend, QPoint(0,0), QPoint(0,height));
        height += item->getHeight() ;
    }
}

void NotifyGroup::resizeEvent(QResizeEvent *event)
{
    QWidget::resizeEvent(event);
}

void NotifyGroup::slotAnimationCloseFinished()
{
    QWidget * w = this->parentWidget();
    if(w && w->layout())
    {
        w->layout()->removeWidget(this);
    }
    this->hide();
    this->deleteLater();

    emit signalGroupClose(m_appID);
}

void NotifyGroup::slotAnimationShowFinished()
{

}

void NotifyGroup::slotAnimationExtendFinished()
{
    m_maskWgt->setVisible(!m_extend);
    this->setFixedHeight(calculateHeight(m_extend));
}

void NotifyGroup::slotTimeout()
{
    if(m_buffItems.isEmpty())
    {
        mTimer->stop();
        return;
    }

    for(int i = 0; i < m_items.count(); i++)
    {
        //当前列表中有添加动画没有执行完的，下一次在添加到layout中
        if(!m_items.at(i)->animationFinished())
        {
            return;
        }
    }

    NotifyItem * item = m_buffItems.takeFirst();
    m_items.append(item);
    //重新计算高度
    this->setFixedHeight(calculateHeight(m_extend));

    m_layout->insertWidget(1,item);

    if(m_items.count() == 1)
    {
        m_title->hide();
        m_maskWgt->hide();
        item->setVisible(true);
        item->slotShow();
    }
    else
    {
        m_title->show();
        for(int i = 0; i < m_items.count() ; i++)
        {
            m_items.at(i)->setVisible(m_extend);
        }
        if(m_extend)
        {
            item->slotShow();
        }
        else
        {
            m_maskWgt->show();
        }
    }

    //将title提升到顶层
    m_title->raise();
    m_title->setNotifyCount(m_items.count());
}

int NotifyGroup::calculateHeight(bool extend)
{
    int height  = 0;
    height += m_title->height();
    if(extend)
    {
//        height += m_layout->spacing();
        for(int i = 0 ; i < m_items.count(); i++)
        {
            NotifyItem * item = m_items.at(i);
            height += item->getHeight();
            height += m_layout->spacing();
        }
    }
    else
    {
        height += MASK_HEIGHT;
    }

    if(m_items.count() == 1)
    {
        height = m_items.at(0)->getHeight();
    }
    return height;
}

