#include "statusnotifiertray.h"
#include "statusnotifierwatcher.h"
#include "statusnotifierpopup.h"
#include "../panel/indepanelplugin.h"
#include "../panel/pluginsettings.h"
#include <QApplication>
#include <QDebug>
#include <QMimeData>
#include <QStringList>
#include <QStylePainter>
#include <QPainter>
#include <QStyleOptionToolButton>
#include <QScreen>

#include "sniasync.h"

static const QString s_watcherServiceName(QStringLiteral("org.kde.StatusNotifierWatcher"));
static const int DEFAULT_TRAY_SIZE = 6;
StatusNotifierArrow::StatusNotifierArrow(QWidget * parent):QToolButton(parent)
{
    setAcceptDrops(true);
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    setAutoRaise(false);
}
void StatusNotifierArrow::dropEvent(QDropEvent *event)
{
    QString  serviceAndPath;
    QDataStream stream(event->mimeData()->data(StatusNotifierWidget::mimeDataFormat()));
    stream >> serviceAndPath;
    emit dropArrow(serviceAndPath);
}
void StatusNotifierArrow::dragEnterEvent(QDragEnterEvent *event)
{
    if (event->mimeData()->hasFormat(mimeDataFormat()))
    {
        event->acceptProposedAction();
    }
}
Qt::Corner StatusNotifierArrow::origin() const
{
    return mOrigin;
}

void StatusNotifierArrow::setOrigin(Qt::Corner newOrigin)
{
    if (mOrigin != newOrigin)
    {
        mOrigin = newOrigin;
        update();
    }
}
void StatusNotifierArrow::paintEvent(QPaintEvent *event)
{
    QSize sz = size();
    QSize adjSz = sz;
    QTransform transform;
    QPoint originPoint;

    switch (mOrigin)
    {
    case Qt::TopLeftCorner:
        transform.rotate(0.0);
        originPoint = QPoint(0.0, 0.0);
        break;

    case Qt::TopRightCorner:
        transform.rotate(90.0);
        originPoint = QPoint(0.0, -sz.width());
        adjSz.transpose();
        break;

    case Qt::BottomRightCorner:
        transform.rotate(180.0);
        originPoint = QPoint(-sz.width(), -sz.height());
        break;

    case Qt::BottomLeftCorner:
        transform.rotate(270.0);
        originPoint = QPoint(-sz.height(), 0.0);
        adjSz.transpose();
        break;
    }
    bool drawPixmapNextTime = false;
    if (!mDrawPixmap)
    {
        mPixmap = QPixmap(adjSz);
        mPixmap.fill(QColor(0, 0, 0, 0));

        if (adjSz != sz)
            resize(adjSz);

        QStylePainter painter(&mPixmap, this);
        QStyleOptionToolButton opt;
        initStyleOption(&opt);
        painter.drawComplexControl(QStyle::CC_ToolButton, opt);

        if (adjSz != sz)
        {
            resize(sz);
            drawPixmapNextTime = true;
        }
        else
            mDrawPixmap = true;
    }
    if (mDrawPixmap)
    {
        QPainter painter(this);
        painter.setTransform(transform);
        painter.drawPixmap(originPoint, mPixmap);

        drawPixmapNextTime = false;
    }
    mDrawPixmap = drawPixmapNextTime;
}
StatusNotifierTray::StatusNotifierTray(INdePanelPlugin *plugin, QWidget *parent) : StatusNotifierWidget(plugin, "trayOrder", parent)
{
    //记录下panel位置
    mCurPanelPos = plugin->panel()->position();

    mPopup = new StatusNotifierPopup(plugin);

    mSettings = plugin->settings();
    mWatchCursor = new QTimer(this);
    mWatchCursor->setInterval(100);
    connect(mWatchCursor, &QTimer::timeout, this, [=]{
//        if(StatusNotifierButton::sDraggging && QCursor::pos().y() <= this->plugin()->panel()->globalGeometry().y())
        if(StatusNotifierButton::sDraggging && qApp->primaryScreen()->availableGeometry().contains(QCursor::pos()))
        {
            mArrow->show();
            mPopup->realign();
            recalculatePopupPosition();
            plugin->panel()->willShowWindow(mPopup);
            mPopup->show();
            mPopup->activateWindow();
            mWatchCursor->stop();
        }
        else {
            hideArrow();
        }
    });
    mArrow = new StatusNotifierArrow (this);
    QString arrowText = "btn_arrow";
    mArrow->setObjectName(arrowText);
    QStringList trayOrder = mSettings->value(this->orderName()).toStringList();
    if(!trayOrder.contains(arrowText))
    {
        trayOrder.push_front(arrowText);
        mSettings->setValue(this->orderName(), trayOrder);
    }
    connect(mArrow, &StatusNotifierArrow::dropArrow, this , &StatusNotifierTray::arrowDropped);
    connect(mArrow, &QToolButton::clicked, this , [=](){
        if(mPopup->isVisible())
            mPopup->hide();
        else
        {
            mPopup->realign();
            recalculatePopupPosition();
            plugin->panel()->willShowWindow(mPopup);
            mPopup->show();
            mPopup->activateWindow();
        }
    });

    layout()->addWidget(mArrow);
    realign();

    hideArrow();

    //处理从任务栏托盘拖动到popup
    connect(mPopup, &StatusNotifierWidget::dropButton, this, [=](const QString & serviceAndPath, int index){
        QStringList trayOrder = mSettings->value(this->orderName()).toStringList();
        QStringList popupOrder = mSettings->value(mPopup->orderName()).toStringList();

        QString id = trayId(serviceAndPath);
        trayOrder.removeAll(id);
        popupOrder.insert(mPopup->index(serviceAndPath), id);
        mSettings->setValue(this->orderName(), trayOrder);
        mSettings->setValue(mPopup->orderName(), popupOrder);
        StatusNotifierButton  *button = layoutRemoved(serviceAndPath);

        button->disconnect();
        connect(button, &StatusNotifierButton::dragging, mPopup, [=] (QObject * dragSource, QPoint const & pos) {
            mPopup->buttonMove(qobject_cast<StatusNotifierButton *>(sender()), qobject_cast<StatusNotifierButton *>(dragSource), pos);
        });
        connect(button, &StatusNotifierButton::dropped, mPopup, [=] (QObject * dragSource, QPoint const & pos, QString & serviceAndPath) {
            mPopup->buttonDropped(qobject_cast<StatusNotifierButton *>(sender()), qobject_cast<StatusNotifierButton *>(dragSource), pos, serviceAndPath);
        });
        mPopup->layoutAdded(serviceAndPath, button, index);

        mPopup->realign();
        QTimer::singleShot(200, this, SLOT(recalculatePopupPosition()));
        hideArrow();
    });
    //处理从popup拖动到任务栏托盘
    connect(this, &StatusNotifierWidget::dropButton, mPopup, [=](const QString & serviceAndPath, int index){
        QStringList trayOrder = mSettings->value(this->orderName()).toStringList();
        QStringList popupOrder = mSettings->value(mPopup->orderName()).toStringList();

        QString id = mPopup->trayId(serviceAndPath);
        popupOrder.removeAll(id);
        trayOrder.insert(this->index(serviceAndPath), id);
        mSettings->setValue(this->orderName(), trayOrder);
        mSettings->setValue(mPopup->orderName(), popupOrder);

        StatusNotifierButton  *button = mPopup->layoutRemoved(serviceAndPath);
        button->disconnect();
        connect(button, &StatusNotifierButton::dragging, this, [=] (QObject * dragSource, QPoint const & pos) {
            buttonMove(qobject_cast<StatusNotifierButton *>(sender()), qobject_cast<StatusNotifierButton *>(dragSource), pos);
        });
        connect(button, &StatusNotifierButton::dropped, this, [=] (QObject * dragSource, QPoint const & pos, QString & serviceAndPath) {
            buttonDropped(qobject_cast<StatusNotifierButton *>(sender()), qobject_cast<StatusNotifierButton *>(dragSource), pos, serviceAndPath);
        });
        this->layoutAdded(serviceAndPath, button, index);
        mPopup->realign();
        QTimer::singleShot(200, this, SLOT(recalculatePopupPosition()));
        hideArrow();
    });

    QString dbusName = QString("org.kde.StatusNotifierHost-%1-%2").arg(QApplication::applicationPid()).arg(1);
    if (!QDBusConnection::sessionBus().registerService(dbusName))
        qDebug() << QDBusConnection::sessionBus().lastError().message();
    mWatcher = new StatusNotifierWatcher(this);
    mWatcher->RegisterStatusNotifierHost(dbusName);

    connect(mWatcher, &StatusNotifierWatcher::StatusNotifierItemRegistered,
            this, [=](const QString & serviceAndPath){
        int slash = serviceAndPath.indexOf('/');
        QString service = serviceAndPath.left(slash);
        QString path = serviceAndPath.mid(slash);
        SniAsync *interface = new SniAsync(service, path, QDBusConnection::sessionBus(), this);
        interface->propertyGetAsync(QLatin1String("Title"), [=] (QString title) {
            interface->propertyGetAsync(QLatin1String("Id"), [=] (QString tid) {
                QString id = title.isEmpty() ? tid : title;
                QStringList trayOrder = mSettings->value(this->orderName()).toStringList().toSet().toList();
                QStringList popupOrder = mSettings->value(mPopup->orderName()).toStringList().toSet().toList();
                if(popupOrder.contains(id))
                {
                    int index = popupOrder.indexOf(id);
                    mPopup->itemAdded(serviceAndPath, interface, index);
                }
                else
                {
                    int index = trayOrder.indexOf(id);
                    if(index < 0)
                    {
                        trayOrder.push_back(id);
                        mSettings->setValue(this->orderName(), trayOrder);
                        this->itemAdded(serviceAndPath, interface);
                    }
                    else
                    {
                        this->itemAdded(serviceAndPath, interface, index);
                    }
                }
                hideArrow();
            });
        });
    });
    connect(mWatcher, &StatusNotifierWatcher::StatusNotifierItemUnregistered, this, [=](const QString & serviceAndPath){
        QTimer::singleShot(500, this, [=]{
            ItemUnregistered(serviceAndPath);
        });
    });
}
StatusNotifierTray::~StatusNotifierTray()
{
    delete mWatcher;
    delete mPopup;
}
void StatusNotifierTray::ItemUnregistered(const QString &serviceAndPath)
{
    QStringList trayOrder = mSettings->value(this->orderName()).toStringList();
    QStringList popupOrder = mSettings->value(mPopup->orderName()).toStringList();
    QString id = trayId(serviceAndPath);
    QString popupId = mPopup->trayId(serviceAndPath);
    if(popupOrder.contains(popupId) || mPopup->contains(serviceAndPath))
    {
        mPopup->itemRemoved(serviceAndPath);
        popupOrder.removeOne(popupId);
        mSettings->setValue(mPopup->orderName(), popupOrder);
    }
    else
    {
        itemRemoved(serviceAndPath);
        trayOrder.removeAll(id);
        mSettings->setValue(this->orderName(), trayOrder);
    }
    hideArrow();
}
void StatusNotifierTray::hideArrow()
{
    if(0 == mPopup->count() && count() < DEFAULT_TRAY_SIZE)
    {
        mPopup->hide();
        mArrow->hide();
    }
    else
        mArrow->show();
}
void StatusNotifierTray::recalculatePopupPosition()
{
    int x_offset = mArrow->rect().center().x();
    int y_offset = mArrow->rect().center().y();
    switch (plugin()->panel()->position())
    {
    case INdePanel::PositionTop:
        x_offset -= mPopup->width()/2;
        y_offset += mArrow->rect().height()/2 + 5;
        break;
    case INdePanel::PositionBottom:
        x_offset -= mPopup->width()/2;
        y_offset -= mArrow->rect().height()/2 + mPopup->height() + 3;
        break;
    case INdePanel::PositionLeft:
        x_offset +=  mArrow->rect().width()/2 + 5;
        y_offset -= mPopup->height()/2;
        break;
    case INdePanel::PositionRight:
        x_offset -=  mArrow->rect().width()/2 + mPopup->width() + 3;
        y_offset -= mPopup->height()/2;
        break;
    }
    QPoint pos = mapToGlobal(QPoint(x_offset, y_offset));
    mPopup->move(pos);
}

void StatusNotifierTray::realign()
{
    Nde::GridLayout *layout = qobject_cast<Nde::GridLayout*>(this->layout());
    layout->setEnabled(false);

    INdePanel *panel = plugin()->panel();
    if (panel->isHorizontal())
    {
        layout->setRowCount(panel->lineCount());
        layout->setColumnCount(0);
    }
    else
    {
        layout->setColumnCount(panel->lineCount());
        layout->setRowCount(0);
    }
    layout->setEnabled(true);

    switch (panel->position())
    {
    case INdePanel::PositionTop:
        mArrow->setOrigin(Qt::BottomRightCorner);
        break;
    case INdePanel::PositionBottom:
        mArrow->setOrigin(Qt::TopLeftCorner);
        break;

    case INdePanel::PositionLeft:
        mArrow->setOrigin(Qt::TopRightCorner);
        break;

    case INdePanel::PositionRight:
        mArrow->setOrigin(Qt::BottomLeftCorner);
        break;
    }

    //panel位置变化或尺寸变化隐藏掉popup
    QRect rc = qApp->primaryScreen()->geometry();
    INdePanel::Position pos = panel->position();
    if(mPopup && (mCurPanelPos != pos ||  mScreenRect != rc))
    {
        mPopup->hide();
        mCurPanelPos = pos;
        mScreenRect = rc;
    }
}

void StatusNotifierTray::leaveEvent(QEvent *event)
{
    mWatchCursor->start();
    QWidget::leaveEvent(event);
}
