/* BEGIN_COMMON_COPYRIGHT_HEADER
 * (c)LGPL2+
 *
 * Nde - a lightweight, Qt based, desktop toolset
 * https://lxqt.org
 *
 * Copyright: 2011 Razor team
 *            2014 Nde team
 * Authors:
 *   Alexander Sokoloff <sokoloff.a@gmail.com>
 *   Maciej Płaza <plaza.maciej@gmail.com>
 *   Kuzma Shapran <kuzma.shapran@gmail.com>
 *
 * This program or library is free software; you can redistribute it
 * and/or modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * END_COMMON_COPYRIGHT_HEADER */

#include "ndegrouppopup.h"
#include <QEnterEvent>
#include <QDrag>
#include <QMimeData>
#include <QLayout>
#include <QPainter>
#include <QStyleOption>
#include <QBitmap>
#include <QDebug>

#include <KWindowEffects>

/************************************************
    this class is just a container of window buttons
    the main purpose is showing window buttons in
    vertical layout and drag&drop feature inside
    group
 ************************************************/
NdeGroupPopup::NdeGroupPopup(NdeTaskGroup *group):
    QFrame(group),
    mGroup(group)
{
    Q_ASSERT(group);
    setAcceptDrops(true);
    setWindowFlags(Qt::FramelessWindowHint | Qt::ToolTip);
    setAttribute(Qt::WA_AlwaysShowToolTips);
    setAttribute(Qt::WA_TranslucentBackground);
    KWindowEffects::enableBlurBehind(winId(), true);
//    QColor backgroundColor = mGroup->plugin()->panel()->backgroundColor();
//    QString color = QString("%1, %2, %3, %4")
//        .arg(backgroundColor.red())
//        .arg(backgroundColor.green())
//        .arg(backgroundColor.blue())
//        .arg(mGroup->plugin()->panel()->opacity() / 100.0);
//    setStyleSheet(QString("#NdeGroupPopup { background-color: rgba(" + color + "); }"));
    mLayout = new Nde::GridLayout;
    setLayout(mLayout);
    mLayout->setSpacing(3);
    mLayout->setMargin(3);

    connect(&mCloseTimer, &QTimer::timeout, this, &NdeGroupPopup::closeTimerSlot);
    mCloseTimer.setSingleShot(true);
    mCloseTimer.setInterval(400);

    QTimer::singleShot(0, [this](){changeLayout();realign();});
}

NdeGroupPopup::~NdeGroupPopup()
{
}

void NdeGroupPopup::realign()
{
    int opacity = mGroup->plugin()->panel()->opacity();
    QColor bColor = mGroup->plugin()->panel()->backgroundColor();

    //test
//    if(opacity == 100 )
//        bColor = Qt::yellow;
//    else {
//        bColor = QColor(0,255,0);
//    }

    if(mOpacity == opacity && mBackgroundColor == bColor )
        return;

    mOpacity = opacity;
    mBackgroundColor = bColor;

    QString color = QString("%1, %2, %3, %4")
            .arg(mBackgroundColor.red())
            .arg(mBackgroundColor.green())
            .arg(mBackgroundColor.blue())
            .arg(opacity / 100.0);
    this->setStyleSheet(QString("#TaskBar NdeGroupPopup { background: rgba(" + color + "); }"));
    update();
}
void NdeGroupPopup::changeLayout()
{
    mLayout->setEnabled(false);
    INdePanel *panel = mGroup->plugin()->panel();
    if (panel->isHorizontal())
    {
        mLayout->setRowCount(1);
        mLayout->setColumnCount(mGroup->visibleButtonsCount());
    }
    else
    {
        mLayout->setColumnCount(1);
        mLayout->setRowCount(mGroup->visibleButtonsCount());
    }
    mLayout->setEnabled(true);
}
void NdeGroupPopup::dropEvent(QDropEvent *event)
{
    qlonglong temp;
    QDataStream stream(event->mimeData()->data(NdeTaskButton::mimeDataFormat()));
    stream >> temp;
    WId window = (WId) temp;

    NdeTaskButton *button = nullptr;
    int oldIndex(0);
    // get current position of the button being dragged
    for (int i = 0; i < layout()->count(); i++)
    {
        NdeTaskButton *b = qobject_cast<NdeTaskButton*>(layout()->itemAt(i)->widget());
        if (b && b->windowId() == window)
        {
            button = b;
            oldIndex = i;
            break;
        }
    }

    if (button == nullptr)
        return;

    int newIndex = -1;
    // find the new position to place it in
    for (int i = 0; i < oldIndex && newIndex == -1; i++)
    {
        QWidget *w = layout()->itemAt(i)->widget();
        if (w && w->pos().y() + w->height() / 2 > event->pos().y())
            newIndex = i;
    }
    const int size = layout()->count();
    for (int i = size - 1; i > oldIndex && newIndex == -1; i--)
    {
        QWidget *w = layout()->itemAt(i)->widget();
        if (w && w->pos().y() + w->height() / 2 < event->pos().y())
            newIndex = i;
    }

    if (newIndex == -1 || newIndex == oldIndex)
        return;

    QVBoxLayout * l = qobject_cast<QVBoxLayout *>(layout());
    l->takeAt(oldIndex);
    l->insertWidget(newIndex, button);
    l->invalidate();

}

void NdeGroupPopup::dragEnterEvent(QDragEnterEvent *event)
{
    event->accept();
    QWidget::dragEnterEvent(event);
}

void NdeGroupPopup::dragLeaveEvent(QDragLeaveEvent *event)
{
    hide(false/*not fast*/);
    QFrame::dragLeaveEvent(event);
}

/************************************************
 *
 ************************************************/
void NdeGroupPopup::leaveEvent(QEvent *event)
{
    mCloseTimer.start();
}

/************************************************
 *
 ************************************************/
void NdeGroupPopup::enterEvent(QEvent *event)
{
    mCloseTimer.stop();
}

void NdeGroupPopup::paintEvent(QPaintEvent *event)
{
    QPainter p(this);
    QStyleOption opt;
    opt.initFrom(this);
    style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
}

void NdeGroupPopup::hide(bool fast)
{
    if (fast)
        close();
    else
        mCloseTimer.start();
}

void NdeGroupPopup::show()
{
    mCloseTimer.stop();
    QFrame::show();
}

void NdeGroupPopup::closeTimerSlot()
{
    bool button_has_dnd_hover = false;
    QLayout* l = layout();
    for (int i = 0; l->count() > i; ++i)
    {
        NdeTaskButton const * const button = dynamic_cast<NdeTaskButton const *>(l->itemAt(i)->widget());
        if (0 != button && button->hasDragAndDropHover())
        {
            button_has_dnd_hover = true;
            break;
        }
    }
    if (!button_has_dnd_hover)
        close();
}
